import numpy as np
import scipy.stats as ss
import scipy.optimize as so
import pandas as pd
import matplotlib.pyplot as plt
import pystan
import seaborn as sb

finnm = input('入力データファイル（*.xlsx）= ')
txtoutnm = input('出力ファイル（*.txt）= ')
foutnm = input('画像保存ファイル（*.png）= ')
txtout = open(txtoutnm, 'w')
#
#      Excelファイルの読み込み
#　　　モジュール xlrd がインストール済みであること
#
xlsx_fl = pd.ExcelFile(finnm)                       #   ExcelファイルからExcelFile型オブジェクトを生成
data_xlsx = pd.read_excel(xlsx_fl,  header = None)  #   ExcelFile型オブジェクトからDataFrame型オブジェクトを生成
data = data_xlsx.values                             #   Dataframe型オブジェクトからnumpyの配列オブジェクトを生成
print('\n', data)

txtout.write('\n入力データファイル: {}\n'.format(finnm))

A_name = data[0]
a = len(A_name)
N = len(data) - 1

X = np.empty((N, a), dtype = float)
print('A = ', A_name)
print('a = ', a)
print('N = ', N)

txtout.write('\nA = {}\n'.format(A_name))
txtout.write('a = {}\n'.format(a))
txtout.write('N = {}\n'.format(N))

for i in range(N):
    for j in range(a):
        X[i][j] = data[i + 1][j]

print(X)
txtout.write('\n{}\n'.format(X))

StanData = {'a': a, 'N': N, 'X': X}


sm = pystan.StanModel(file = 'OneFctrRndm.stan')
fit = sm.sampling(data = StanData, n_jobs = 1, iter = 5000, warmup = 3000)
print(fit)

txtout.write('\n{}\n'.format(fit))

#       日本語表記の設定
plt.rcParams['font.family'] = ['Yu Gothic', 'Noto Sans CJK JP', 'Hiragino Maru Gothic Pro']

v_mean = fit['sgm_tau'].mean()
v_025p, v_Q1, v_med, v_Q3, v_975p = np.percentile(fit['sgm_tau'],
                                                 [2.5, 25, 50, 75, 97.5])

print('\n変量効果の標準偏差：')
print('    平均値 = {0:.3f}'.format(v_mean))
print('    中央値 = {0:.3f}'.format(v_med))
print('    Q1 = {0:.3f}    Q3 = {1:.3f}'.format(v_Q1, v_Q3))
print('    95% CI = [{0:.3f}, {1:.3f}]'.format(v_025p, v_975p))

txtout.write('\n\n変量効果の標準偏差\n')
txtout.write('    平均値 = {0:.3f}\n'.format(v_mean))
txtout.write('    中央値 = {0:.3f}\n'.format(v_med))
txtout.write('    Q1 = {0:.3f}    Q3 = {1:.3f}\n'.format(v_Q1, v_Q3))
txtout.write('    95% CI = [{0:.3f}, {1:.3f}]\n'.format(v_025p, v_975p)) 

plt.title('変量効果の標準偏差の事後分布' + \
          '\n平均値 = {0:.3f}    中央値 = {1:.3f}'.format(v_mean, v_med), fontsize = 20)
sb.kdeplot(fit['sgm_tau'], clip = (0.0, np.percentile(fit['sgm_tau'], 99)))
plt.yticks([])
plt.tight_layout()
plt.savefig(foutnm[:-4] + '_sgm_tau.png')
savedFiles = [foutnm[:-4] + '_sgm_tau.png']
plt.show()

for j in range(a):
    print('\n水準{}:'.format(A_name[j]))
    txtout.write('\n\n水準{}:\n'.format(A_name[j]))
    v_mean = fit['f_a'][:, j].mean()
    v_025p, v_Q1, v_med, v_Q3, v_975p = np.percentile(fit['f_a'][:, j],
                                                     [2.5, 25, 50, 75, 97.5])
 
    print('    平均値 = {0:.3f}'.format(v_mean))
    print('    中央値 = {0:.3f}'.format(v_med))
    print('    Q1 = {0:.3f}    Q3 = {1:.3f}'.format(v_Q1, v_Q3))
    print('    95% CI = [{0:.3f}, {1:.3f}]'.format(v_025p, v_975p))
   
    txtout.write('    平均値 = {0:.3f}\n'.format(v_mean))
    txtout.write('    中央値 = {0:.3f}\n'.format(v_med))
    txtout.write('    Q1 = {0:.3f}    Q3 = {1:.3f}\n'.format(v_Q1, v_Q3))
    txtout.write('    95% CI = [{0:.3f}, {1:.3f}]\n'.format(v_025p, v_975p)) 

plt.figure(figsize = (8,5))
plt.title('水準の値', fontsize = 20)
for j in range(a):
    sb.kdeplot(fit['f_a'][:,j], label = A_name[j])
plt.yticks([])
plt.legend()
plt.savefig(foutnm[:-4] + '_fa.png')
savedFiles.append(foutnm[:-4] + '_fa.png')
plt.show()

effcts = []
for st, sd in zip(fit['sgm_tau'], fit['sgm']):
    effcts.append(st / sd)

effcts = np.array(effcts)
print('\n効果量:')
txtout.write('\n\n効果量:\n')
v_mean = effcts.mean()
v_025p, v_Q1, v_med, v_Q3, v_975p = np.percentile(effcts, [2.5, 25, 50, 75, 97.5])

print('    平均値 = {0:.3f}'.format(v_mean))
print('    中央値 = {0:.3f}'.format(v_med))
print('    Q1 = {0:.3f}    Q3 = {1:.3f}'.format(v_Q1, v_Q3))
print('    95% CI = [{0:.3f}, {1:.3f}]'.format(v_025p, v_975p))

txtout.write('    平均値 = {0:.3f}\n'.format(v_mean))
txtout.write('    中央値 = {0:.3f}\n'.format(v_med))
txtout.write('    Q1 = {0:.3f}    Q3 = {1:.3f}\n'.format(v_Q1, v_Q3))
txtout.write('    95% CI = [{0:.3f}, {1:.3f}]\n'.format(v_025p, v_975p))

plt.title('効果量' + \
          '\n平均値 = {0:.2f}   中央値 = {1:.2f}'.format(v_mean, v_med), fontsize = 20)
sb.kdeplot(effcts, clip = (0.0, np.percentile(effcts, 99)))
plt.yticks([])
plt.tight_layout()
plt.savefig(foutnm[:-4] + '_es.png')
savedFiles.append(foutnm[:-4] + '_es.png')
plt.show()
   
txtout.close()
savedFiles.append(txtoutnm)

print('\n保存ファイル:')
for nm in savedFiles:
    print('    ', nm)
print()
